﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Lab1;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningYborshik
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityYborshik? EntityYborshik { get; private set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    private int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    private int? _startPosY;

    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    private readonly int _drawningCarWidth = 140;

    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    private readonly int _drawningCarHeight = 70;

    /// <summary>
    /// Инициализация полей объекта-класса спортивного автомобиля
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="flashingLights">Признак наличия мигалок</param>
    /// <param name="ladle">Признак наличия ковша</param>

    public void Init(int speed, int weight, Color bodyColor, Color additionalColor, bool flashingLights, bool ladle)
    {
        EntityYborshik = new EntityYborshik();
        EntityYborshik.Init(speed, weight, bodyColor, additionalColor, flashingLights, ladle);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
	/// Установка границ поля
	/// </summary>
	/// <param name="width">Ширина поля</param>
	/// <param name="height">Высота поля</param>
	/// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
	public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
        if (_drawningCarWidth < width || _drawningCarHeight < height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            return true;
        }
        else
            return false;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        if (x > 0 && y > 0 && _drawningCarWidth < _pictureWidth && _drawningCarHeight < _pictureHeight)
        {
            _startPosX = x;
            _startPosY = y;
        }
        else {
            Random rnd = new Random();
            _startPosX = rnd.Next(0, _pictureWidth.Value - _drawningCarWidth);
            _startPosY = rnd.Next(0, _pictureHeight.Value - _drawningCarHeight);

        }
    }
    /// <summary>
	/// Изменение направления перемещения
	/// </summary>
	/// <param name="direction">Направление</param>
	/// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
	public bool MoveTransport(DirectionType direction)
    {
        if (EntityYborshik == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityYborshik.Step > 0)
                {
                    _startPosX -= (int)EntityYborshik.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityYborshik.Step > 0)
                {
                    _startPosY -= (int)EntityYborshik.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                //TODO прописать логику сдвига в право
                if (_startPosX.Value + EntityYborshik.Step < _pictureWidth - _drawningCarWidth)
                {
                    _startPosX += (int)EntityYborshik.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                //TODO прописать логику сдвига в вниз
                if (_startPosY.Value + EntityYborshik.Step < _pictureHeight - _drawningCarHeight)
                {
                    _startPosY += (int)EntityYborshik.Step;
                }
                return true;
            default:
                return false;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityYborshik == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        
        Brush additionalBrush = new SolidBrush(EntityYborshik.AdditionalColor);
        Pen additionalPen = new(EntityYborshik.AdditionalColor);
        // баки
        if (EntityYborshik.Ladle)
        {
            g.FillEllipse(additionalBrush, _startPosX.Value + 10, _startPosY.Value, 105, 32);
        }
        // щетка
        if (EntityYborshik.FlashingLights)
        {
            g.DrawLine(additionalPen, _startPosX.Value + 80, _startPosY.Value + 50, _startPosX.Value + 80, _startPosY.Value + 75);
            g.DrawLine(additionalPen, _startPosX.Value + 80, _startPosY.Value + 50, _startPosX.Value + 90, _startPosY.Value + 75);
            g.DrawLine(additionalPen, _startPosX.Value + 80, _startPosY.Value + 50, _startPosX.Value + 100, _startPosY.Value + 75);
            g.DrawLine(additionalPen, _startPosX.Value + 80, _startPosY.Value + 50, _startPosX.Value + 110, _startPosY.Value + 75);
            g.DrawLine(additionalPen, _startPosX.Value + 80, _startPosY.Value + 50, _startPosX.Value + 120, _startPosY.Value + 75);
        }
        Brush br = new SolidBrush(EntityYborshik.BodyColor);

        //кузов уборщика
        g.FillRectangle(br, _startPosX.Value + 10, _startPosY.Value + 30, 140, 20);
        g.FillRectangle(br, _startPosX.Value + 115, _startPosY.Value, 35, 30);
        //колеса уборщика
        g.FillEllipse(br, _startPosX.Value + 15, _startPosY.Value + 50, 25, 25);
        g.FillEllipse(br, _startPosX.Value + 120, _startPosY.Value + 50, 25, 25);
        g.FillEllipse(br, _startPosX.Value + 50, _startPosY.Value + 50, 25, 25);

    }
}


